"""
.. include:: ../_README_temp.md
"""


from ._version import __version__ as version
from .system import System
from .constants import *
from . import systems
from . import rotation
from . import potential
from . import solve
from . import plot

